/*
*  addressbkdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "addressbkdlgs.h"
#include "utility.h"
#include "global.h"
#include "contactdlgs.h"
#include "msgmans.h"
#include "qsdbs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TAddressBkDlg *AddressBkDlg;
//---------------------------------------------------------------------------

bool __fastcall ManageAddressBook(TForm *Owner)
{
  TAddressBkDlg *addressbk = new TAddressBkDlg(Owner);
  addressbk->ShowModal();
  bool ok = addressbk->ModalResult == mrOk;
  delete addressbk;
  return ok;
}
//---------------------------------------------------------------------------

bool __fastcall SelectAddresses(TForm *Owner, TStrings *Msg)
{
   TAddressBkDlg *addressbk = new TAddressBkDlg(Owner, Msg);
   addressbk->ShowModal();
   bool ok = addressbk->ModalResult == mrOk;
   delete addressbk;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TAddressBkDlg::TAddressBkDlg(TComponent* Owner)
        : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "addrsbkdlg", ini);
   orgwin = IntToStr(Width)+IntToStr(Height);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(ContactBox->Font, ini->ReadString("settings", "windowfont",
      OptsDefWindowFont));
   delete ini;

   Caption = "Address Book";
   DB->LoadContacts(ContactBox->Items);

   OkBtn->Caption = "Done";
   OkBtn->Left = CancelBtn->Left;
   CancelBtn->Hide();
   ContactBox->Height = Bc->Top + Bc->Height - ContactBox->Top;
   ToBtn->Enabled = false;
   To->Enabled = false;
   CcBtn->Enabled = false;
   Cc->Enabled = false;
   BcBtn->Enabled = false;
   Bc->Enabled = false;
   mode = addrsManage;
}
//---------------------------------------------------------------------------

__fastcall TAddressBkDlg::TAddressBkDlg(TComponent* Owner, TStrings *Msg)
        : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "addrsbkdlg", ini);
   orgwin = IntToStr(Width)+IntToStr(Height);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(ContactBox->Font, ini->ReadString("settings", "windowfont",
      OptsDefWindowFont));
   To->Font->Color = ContactBox->Font->Color;
   Cc->Font->Color = To->Font->Color;
   Bc->Font->Color = Cc->Font->Color;
   delete ini;

   Caption = "Select Recipients";
   DB->LoadContacts(ContactBox->Items);

   message = Msg;
   msg = new TMsgManager(message);
   mode = addrsSelect;
}
//---------------------------------------------------------------------------

__fastcall TAddressBkDlg::~TAddressBkDlg()
{
   if (IntToStr(Width)+IntToStr(Height) != orgwin)
   {
      TIniFile *ini = new TIniFile(AppIni());
      SaveWindowPosition(this, "addrsbkdlg", ini);
      delete ini;
   }

   delete msg;
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::AddAddress(TMemo *Edit, AnsiString Address)
{
   if (Edit == To)
   {
      Edit->Lines->BeginUpdate();
      if (!To->Text.IsEmpty())
      {
         ContactBox->Height = ContactBox->Height - (orgtoheight - 10);
         ToBtn->Top = ToBtn->Top - (orgtoheight - 10);
         To->Top = ToBtn->Top;
         To->Height = To->Height + orgtoheight - 10;
      }
      To->Lines->Add(Address);
      Edit->SelStart = 0;
      Edit->Perform(EM_SCROLLCARET, 0, 0);
      Edit->Lines->EndUpdate();
      return;
   }
   if (Edit == Cc)
   {
      Edit->Lines->BeginUpdate();
      if (!Cc->Text.IsEmpty())
      {
         ContactBox->Height = ContactBox->Height - (orgtoheight - 10);
         ToBtn->Top = ToBtn->Top - (orgtoheight - 10);
         To->Top = ToBtn->Top;
         CcBtn->Top = CcBtn->Top - (orgtoheight - 10);
         Cc->Top = CcBtn->Top;
         Cc->Height = Cc->Height + orgtoheight - 10;
      }
      Cc->Lines->Add(Address);
      Edit->SelStart = 0;
      Edit->Perform(EM_SCROLLCARET, 0, 0);
      Edit->Lines->EndUpdate();
   }
   if (Edit == Bc)
   {
      Edit->Lines->BeginUpdate();
      if (!Bc->Text.IsEmpty())
      {
         ContactBox->Height = ContactBox->Height - (orgtoheight - 10);
         ToBtn->Top = ToBtn->Top - (orgtoheight - 10);
         To->Top = ToBtn->Top;
         CcBtn->Top = CcBtn->Top - (orgtoheight - 10);
         Cc->Top = CcBtn->Top;
         BcBtn->Top = BcBtn->Top - (orgtoheight - 10);
         Bc->Top = BcBtn->Top;
         Bc->Height = Bc->Height + orgtoheight - 10;
      }
      Bc->Lines->Add(Address);
      Edit->SelStart = 0;
      Edit->Perform(EM_SCROLLCARET, 0, 0);
      Edit->Lines->EndUpdate();
   }
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (mode == addrsManage)
      return;

   if (ModalResult == mrOk)
   {
      int x, y;
      AnsiString hdr, addr;
      if (To->Text != "")
      {
         hdr = "To: ";
         x = To->Lines->Count;
         for (int i = 0; i < x; i++)
         {
            if (i > 0)
               hdr += " ";
            hdr += To->Lines->Strings[i];
            if (i < x - 1)
               hdr += ",\r\n";
         }
         x = msg->DelHeader("To:");
         if (x < 0)
            x = msg->EOH();
         message->Insert(x, hdr);
      }
      else
      {
         if ((x = msg->IndexOfHeader("To:")) >= 0)
            message->Strings[x] = "To: ";
      }

      if (Cc->Text != "")
      {
         hdr = "Cc: ";
         x = Cc->Lines->Count;
         for (int i = 0; i < x; i++)
         {
            if (i > 0)
               hdr += " ";
            hdr += Cc->Lines->Strings[i];
            if (i < x - 1)
               hdr += ",\r\n";
         }
         x = msg->DelHeader("Cc:");
         if (x < 0)
            x = msg->IndexOfHeaderAfter("To:");
         if (x < 0)
            x = msg->EOH();
         message->Insert(x, hdr);
      }
      else
         msg->DelHeader("Cc:");

      if (Bc->Text != "")
      {
         hdr = "Bcc: ";
         x = Bc->Lines->Count;
         for (int i = 0; i < x; i++)
         {
            if (i > 0)
               hdr += " ";
            hdr += Bc->Lines->Strings[i];
            if (i < x - 1)
               hdr += ",\r\n";
         }
         x = msg->DelHeader("Bcc:");
         if (x < 0)
            x = msg->IndexOfHeaderAfter("Cc:");
         if (x < 0)
            x = msg->IndexOfHeaderAfter("To:");
         if (x < 0)
            x = msg->EOH();
         message->Insert(x, hdr);
      }
      else
         msg->DelHeader("Bcc:");
   }
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::ToBtnClick(TObject *Sender)
{
   assert(ContactBox->ItemIndex >= 0);
   AddAddress(To, ContactBox->Items->Strings[ContactBox->ItemIndex]);
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::CcBtnClick(TObject *Sender)
{
   assert(ContactBox->ItemIndex >= 0);
   AddAddress(Cc, ContactBox->Items->Strings[ContactBox->ItemIndex]);
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::BcBtnClick(TObject *Sender)
{
   assert(ContactBox->ItemIndex >= 0);
   AddAddress(Bc, ContactBox->Items->Strings[ContactBox->ItemIndex]);
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::ContactBoxClick(TObject *Sender)
{
   EditBtn->Enabled = ContactBox->ItemIndex >= 0;
   DeleteBtn->Enabled = EditBtn->Enabled;
   ToBtn->Enabled = EditBtn->Enabled;
   CcBtn->Enabled = ToBtn->Enabled;
   BcBtn->Enabled = CcBtn->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::PopupMenu1Popup(TObject *Sender)
{
   Edit1->Enabled = ContactBox->ItemIndex >= 0;
   Delete1->Enabled = Edit1->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::Resizedialog1Click(TObject *Sender)
{
   BorderStyle = bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::Locksize1Click(TObject *Sender)
{
   BorderStyle = bsDialog;
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::New1Click(TObject *Sender)
{
   AnsiString email = NewContactProfile(this);
   if (!email.IsEmpty())
   {
      ContactBox->ItemIndex = ContactBox->Items->Add(email);
   }
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::Edit1Click(TObject *Sender)
{
   assert(ContactBox->ItemIndex >= 0);
   AnsiString newemail, oldemail = ContactBox->Items->Strings[ContactBox->ItemIndex];
   newemail = EditContactProfile(this, oldemail);
   if (!newemail.IsEmpty() && newemail != oldemail)
   {
      ContactBox->Items->Delete(ContactBox->ItemIndex);
      ContactBox->ItemIndex = ContactBox->Items->Add(newemail);
   }
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::Delete1Click(TObject *Sender)
{
   assert(ContactBox->ItemIndex >= 0);
   if (MsgBox(this,
      "Delete the current contact. Are you sure?   ",
      MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
         return;

   DB->DelContact(ContactBox->Items->Strings[ContactBox->ItemIndex]);
   ContactBox->Items->Delete(ContactBox->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::FormShow(TObject *Sender)
{
   if (mode == addrsManage)
      return;

   orgtoheight = -To->Font->Height + 12;
   To->Height = orgtoheight;
   Cc->Height = orgtoheight;
   Bc->Height = orgtoheight;

   To->Top = ToBtn->Top + 1;
   Cc->Top = CcBtn->Top + 1;
   Bc->Top = BcBtn->Top + 1;
   Cc->Width = To->Width;
   Bc->Width = Cc->Width;

   AnsiString addr;
   TStrings *addrs = new TStringList();
   msg->GetFullAddresses("To:", addrs);
   for (int i = 0; i < addrs->Count; i++)
   {
      AddAddress(To, addrs->Strings[i]);
   }
   addrs->Clear();
   msg->GetFullAddresses("Cc:", addrs);
   for (int i = 0; i < addrs->Count; i++)
   {
      AddAddress(Cc, addrs->Strings[i]);
   }
   addrs->Clear();
   msg->GetFullAddresses("Bcc:", addrs);
   for (int i = 0; i < addrs->Count; i++)
   {
      AddAddress(Bc, addrs->Strings[i]);
   }

   delete addrs;
}
//---------------------------------------------------------------------------

void __fastcall TAddressBkDlg::Open1Click(TObject *Sender)
{
   if (mode == addrsManage)
      Edit1->Click();
   else
      ToBtn->Click();
}
//---------------------------------------------------------------------------

