/*
*  capsdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <clipbrd.hpp>

#include "capsdlgs.h"
#include "mixmaster.h"
#include "utility.h"
#include "help.h"
#include "global.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//TCapsDlg *CapsDlg;
//---------------------------------------------------------------------------

void __fastcall RunCapsViewer(TForm *Owner)
{
   TCapsDlg *capsdlg = new TCapsDlg(Owner);
   capsdlg->ShowModal();
   delete capsdlg;
}
//---------------------------------------------------------------------------

__fastcall TCapsDlg::TCapsDlg(TComponent* Owner)
        : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "capsdlg", ListView, ini);
   orgwin = IntToStr(Width)+IntToStr(Height)+
	  IntToStr(ListView->Column[0]->Width)+
	  IntToStr(ListView->Column[1]->Width)+
	  IntToStr(ListView->Column[2]->Width)+
	  IntToStr(ListView->Column[3]->Width);
#ifndef QS2
   bool expertmode = ini->ReadBool("settings", "expertmode", false);
#else
   bool expertmode = true;
#endif
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(ListBox2->Font, ini->ReadString("settings", "windowfont",
	  OptsDefDialogFont));
   ListView->Font->Assign(ListBox2->Font);
   delete ini;
   //ListView->ShowColumnHeaders = false;

   if (!expertmode)
   {
	  int i = 0; while (i < ListBox2->Items->Count)
	  {
		 if (ListBox2->Items->Strings[i].Pos(" ") == 1)
		 {
			ListBox2->Items->Delete(i);
			continue;
		 }
		 i++;
	  }
   }

   type2 = new TStringList();
   mlist = new TStringList();
   klist = new TStringList();
   rlist = new TStringList();
   broke = new TStringList();

   MixGetType2List(type2);

   if (FileExists(MixMListTxtFile()))
	  mlist->LoadFromFile(MixMListTxtFile());
   bool saving = false;
   for (int i = 0; i < mlist->Count; i++)
   {
	  AnsiString line = mlist->Strings[i];
	  if (saving)
	  {
		 if (line.Pos("(") != 1)
			break;
		 broke->Add(line);
		 continue;
	  }
	  if (line.Pos("Broken type-II") == 1)
		 saving = true;
   }

   int i = 0; while (i < mlist->Count)
   {
	  AnsiString line = mlist->Strings[i];
	  if (line.Pos("Last update:") == 1)
		 Caption = Caption + " - " + line;
	  if (line.Pos("%") == 0)
	  {
		 mlist->Delete(i);
		 continue;
	  }
	  i++;
   }

   if (FileExists(MixRListTxtFile()))
	  rlist->LoadFromFile(MixRListTxtFile());
   i = 0; while (i < rlist->Count)
   {
	  if (rlist->Strings[i].Pos("$remailer{") == 0)
	  {
		 rlist->Delete(i);
		 continue;
	  }
	  i++;
   }

   TStringList *mring = new TStringList();
   if (FileExists(MixPubringMixFile()))
	  mring->LoadFromFile(MixPubringMixFile());
   i = 0; for (i = 0; i < mring->Count; i++)
   {
	  AnsiString rem, line = mring->Strings[i];
	  if (line.Pos("@") > 0)
	  {
		 rem = GetTok(0, line, ' ');
		 i += 4;
		 if (mring->Strings[i] == "1026")
			klist->Add(rem + " 4096");
		 else
			klist->Add(rem + " 1024");
	  }
   }
   delete mring;

   Label3->Caption = "";
   Label2->Caption = "";
   ListBox2->ItemIndex = 0;
   ListBox2Click(NULL);
}
//---------------------------------------------------------------------------

__fastcall TCapsDlg::~TCapsDlg()
{
   if (IntToStr(Width)+IntToStr(Height)+
	   IntToStr(ListView->Column[0]->Width)+
	   IntToStr(ListView->Column[1]->Width)+
	   IntToStr(ListView->Column[2]->Width)+
	   IntToStr(ListView->Column[3]->Width) != orgwin)
   {
	  TIniFile *ini = new TIniFile(AppIni());
	  SaveWindowPosition(this, "capsdlg", ListView, ini);
	  delete ini;
   }

   delete broke;
   delete rlist;
   delete klist;
   delete mlist;
   delete type2;
}
//---------------------------------------------------------------------------

void __fastcall TCapsDlg::ListBox2Click(TObject *Sender)
{
   int x = ListBox2->ItemIndex;
   if (x < 0)
	  return;

   /*if (ListView->Column[1]->Caption != "History")
   {
	  ListView->Column[1]->Caption = "History";
   }
   else if (ListView->Column[0]->Caption == "Broken Chains")
   {
	  ListView->Column[0]->Caption = "Remailer";
	  ListView->Column[1]->Caption = "History";
	  ListView->Column[2]->Caption = "Latency";
	  ListView->Column[3]->Caption = "Uptime";
   }*/

   ListView->Column[0]->Caption = "Remailer";
   ListView->Column[1]->Caption = "History";
   ListView->Column[2]->Caption = "Latency";
   ListView->Column[3]->Caption = "Uptime";

   ListView->Items->Clear();
   Label3->Caption = "";
   AnsiString cap = ListBox2->Items->Strings[x].Trim();

   ListView->Items->BeginUpdate();
   if (cap == "all mix")
   {
	  for (int i = 0; i < mlist->Count; i++)
	  {
		 ListViewAdd(mlist->Strings[i]);
	  }
   }
   else if (cap == "middleman")
   {
	  for (int i = 0; i < mlist->Count; i++)
	  {
		 AnsiString rem, line, mline = mlist->Strings[i];
		 rem = GetTok(0, mline, ' ');
		 for (int t = 0; t < type2->Count; t++)
		 {
			line = type2->Strings[t];
			if (GetTok(0, line, ' ') == rem && GetTok(4, line, ' ').Pos("M") > 0)
			   ListViewAdd(mline);
		 }
	  }
   }
   else if (cap == "exit")
   {
	  for (int i = 0; i < mlist->Count; i++)
	  {
		 AnsiString rem, line, mline = mlist->Strings[i];
		 rem = GetTok(0, mline, ' ');
		 for (int t = 0; t < type2->Count; t++)
		 {
			line = type2->Strings[t];
			if (GetTok(0, line, ' ') == rem && GetTok(4, line, ' ').Pos("M") == 0)
			   ListViewAdd(mline);
		 }
	  }
   }
   else if (cap == "post")
   {
	  for (int i = 0; i < mlist->Count; i++)
      {
         AnsiString rem, line, mline = mlist->Strings[i];
         rem = GetTok(0, mline, ' ');
		 for (int t = 0; t < type2->Count; t++)
		 {
            line = type2->Strings[t];
			if (GetTok(0, line, ' ') == rem && GetTok(4, line, ' ').Pos("N") > 0)
			   ListViewAdd(mline);
         }
      }
   }
   else if (cap == "via local inews")
   {
      for (int i = 0; i < mlist->Count; i++)
	  {
         AnsiString rem, line, mline = mlist->Strings[i];
         rem = GetTok(0, mline, ' ');
         for (int t = 0; t < type2->Count; t++)
         {
            line = type2->Strings[t];
            if (GetTok(0, line, ' ') == rem && GetTok(4, line, ' ').Pos("p") > 0)
               ListViewAdd(mline);
         }
	  }
   }
   else if (cap == "via mail2news")
   {
      for (int i = 0; i < mlist->Count; i++)
      {
         AnsiString rem, line, mline = mlist->Strings[i];
		 rem = GetTok(0, mline, ' ');
		 for (int t = 0; t < type2->Count; t++)
         {
            line = type2->Strings[t];
            if (GetTok(0, line, ' ') == rem && GetTok(4, line, ' ').Pos("m") > 0)
			   ListViewAdd(mline);
		 }
      }
   }
   else if (cap == "hybrid")
   {
	  for (int i = 0; i < mlist->Count; i++)
      {
		 AnsiString mline = mlist->Strings[i], rem = GetTok(0, mline, ' ');
		 for (int i = 0; i < rlist->Count; i++)
         {
			AnsiString rline = rlist->Strings[i];
            if (rline.Pos("\""+rem+"\"") > 0 && rline.Pos("hybrid") > 0)
            {
               ListViewAdd(mline);
               break;
            }
         }
      }
   }
   else if (cap == "version")
   {
      ListView->Column[1]->Caption = "Version";
	  for (int i = 0; i < mlist->Count; i++)
      {
		 AnsiString mline = mlist->Strings[i], rem = GetTok(0, mline, ' ');
		 for (int i = 0; i < type2->Count; i++)
		 {
			AnsiString ver, tline = type2->Strings[i];
            if (tline.Pos(rem+" ") == 1)
            {
               ver = GetTok(3, tline, ' ').SubString(1, 12);
			   while (ver.Length() < 12)
				  ver += " ";
               mline.Delete(16, 12);
			   mline.Insert(ver, 16);
               ListViewAdd(mline);
			   break;
            }
         }
	  }
   }
   else if (cap == "max size")
   {
	  ListView->Column[1]->Caption = "       Max Size";
	  Label3->Caption = "If a remailer isn't listed, it's max size is unknown.   '----' = no limit.";
	  for (int i = 0; i < mlist->Count; i++)
	  {
		 AnsiString mline = mlist->Strings[i], rem = GetTok(0, mline, ' ');
		 for (int i = 0; i < rlist->Count; i++)
		 {
			AnsiString klen, rline = rlist->Strings[i];
			if (rline.Pos("\""+rem+"\"") > 0)
			{
			   int x = rline.Pos("klen");
			   AnsiString value, klen, nums = "0123456789";
			   if (x == 0)
			   {
				  klen = "---- ";
			   }
			   else
			   {
				  klen = rline.SubString(x + 4, 6);
				  while (klen.Length() && nums.Pos(klen[klen.Length()]) == 0)
					 klen.SetLength(klen.Length() - 1);
				  klen += "K ";
			   }
			   while (klen.Length() < 10)
				  klen = " " + klen;
			   klen += " ";
			   mline.Delete(16, 12);
			   mline.Insert(klen, 16);
			   ListViewAddK(mline);
			   break;
			}
		 }
	  }
   }
   else if (cap == "broken chains")
   {
	  ListView->Column[0]->Caption = "Broken Chains";
	  ListView->Column[1]->Caption = "";
	  ListView->Column[2]->Caption = "";
	  ListView->Column[3]->Caption = "";
	  for (int i = 0; i < broke->Count; i++)
	  {
		 AnsiString chain = broke->Strings[i];
		 AnsiString from = GetTok(0, chain, ' '), to = GetTok(1, chain, ' ');
		 TListItem *item = ListView->Items->Add();
		 item->Caption = from.SubString(2, from.Length() - 1).Trim();
		 item->SubItems->Add(to.SubString(1, to.Length() - 1).Trim());
	  }
   }
   else if (cap == "4096 keys")
   {
	  for (int i = 0; i < mlist->Count; i++)
	  {
		 AnsiString rem, line = mlist->Strings[i];
		 rem = GetTok(0, line, ' ');
		 if (klist->IndexOf(rem+" 4096") >= 0)
		 {
			ListViewAdd(line);
		 }
	  }
   }
   else if (cap == "1024 keys")
   {
	  for (int i = 0; i < mlist->Count; i++)
	  {
		 AnsiString rem, line = mlist->Strings[i];
		 rem = GetTok(0, line, ' ');
		 if (klist->IndexOf(rem+" 1024") >= 0)
		 {
			ListViewAdd(line);
		 }
	  }
   }

   ListView->Items->EndUpdate();

   int c = ListView->Items->Count;
   Label2->Caption = IntToStr(c) + (c == 1 ? " remailer" : " remailers ");
   return;
}
//---------------------------------------------------------------------------

void __fastcall TCapsDlg::ListViewAdd(const AnsiString &line)
{
   TListItem *item = ListView->Items->Add();
   item->Caption = line.SubString(1, 15). Trim();
   item->SubItems->Add(line.SubString(16, 12).Trim());
   item->SubItems->Add(line.SubString(28, 9).Trim());
   item->SubItems->Add(line.SubString(37, 8).Trim());
}
//---------------------------------------------------------------------------

void __fastcall TCapsDlg::ListViewAddK(const AnsiString &line)
{
   TListItem *item = ListView->Items->Add();
   item->Caption = line.SubString(1, 15). Trim();
   item->SubItems->Add(line.SubString(16, 12));
   item->SubItems->Add(line.SubString(28, 9).Trim());
   item->SubItems->Add(line.SubString(37, 8).Trim());
}
//---------------------------------------------------------------------------

void __fastcall TCapsDlg::PopupMenu1Popup(TObject *Sender)
{
   Copyname1->Enabled = ListView->ItemIndex >= 0;
   Resizedialog1->Enabled = BorderStyle == bsDialog;
   Locksize1->Enabled = BorderStyle == bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TCapsDlg::Copyname1Click(TObject *Sender)
{
   Clipboard()->AsText = ListView->Selected->Caption;
}
//---------------------------------------------------------------------------

void __fastcall TCapsDlg::Resizedialog1Click(TObject *Sender)
{
   //ListView->ShowColumnHeaders = true;
   BorderStyle = bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TCapsDlg::Locksize1Click(TObject *Sender)
{
   //ListView->ShowColumnHeaders = false;
   BorderStyle = bsDialog;
}
//---------------------------------------------------------------------------

void __fastcall TCapsDlg::Legend1Click(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/mlistlegend.htm>legendwin");
}
//---------------------------------------------------------------------------

void __fastcall TCapsDlg::ListViewAdvancedCustomDrawSubItem(
	  TCustomListView *Sender, TListItem *Item, int SubItem,
      TCustomDrawState State, TCustomDrawStage Stage, bool &DefaultDraw)
{
   if (SubItem == 1)
      ListView->Canvas->Font->Name = "courier new";
}
//---------------------------------------------------------------------------

