/*
*  decodedlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "decodedlgs.h"
#include "utility.h"
#include "global.h"
#include "mimedecs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDecodeWin *DecodeWin;
const AnsiString decodemsg =
"     QS places PGPed attachments inline in the message body. If your email "
"program doesn't recognize these inline attachments you can use this feature to "
"decode them. To do so, save the message to disk. Include at least one of the "
"message headers. Attachments will be saved in the same directory as the file "
"you've saved. \n\n"
"     This software can also decode multipart MIME messages. In that case, all "
"MIME related message headers must be included.";

//---------------------------------------------------------------------------

void __fastcall DecodeMIMEInlineAttachments(TForm *Owner)
{
   TDecodeWin *decodewin = new TDecodeWin(Owner);
   decodewin->ShowModal();
   delete decodewin;
}
//---------------------------------------------------------------------------

__fastcall TDecodeWin::TDecodeWin(TComponent* Owner)
        : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   ChDir(AppDir());
   OpenDialog->InitialDir = AppDir();
   RichEdit1->Lines->Text = decodemsg;
}
//---------------------------------------------------------------------------

void __fastcall TDecodeWin::Browse1Click(TObject *Sender)
{
   if (OpenDialog->Execute())
   {
      FileName->Text = OpenDialog->FileName;
      DecodeBtn->Enabled = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TDecodeWin::Decode1Click(TObject *Sender)
{
   if (MsgBox(this,
      "Decode the messages attachments now? ",
       MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
   {
      return;
   }

   if (!FileExists(FileName->Text))
   {
      MsgBox(this,
         "The specified file doesn't exist! ",
          MT_ERROR, MB_OK);
      return;
   }

   TStringList *msg = new TStringList();
   TStringList *filenames = new TStringList();

   msg->LoadFromFile(FileName->Text);
   DecodeMIMEMessage(this, msg, ExtractFilePath(FileName->Text), filenames);

   Label1->Enabled = false;
   FileName->Enabled = false;
   DecodeBtn->Enabled = false;
   BrowseBtn->Enabled = false;
   OpenBtn->Enabled = false;
   Snooze(500);

   AnsiString resultmsg;
   if (filenames->Count)
      resultmsg = "These files were found and decoded: \n\n" + filenames->Text;
   else
      resultmsg = "Sorry, no attachments were found! ";
   MsgBox(this, resultmsg, MT_INFO, MB_OK);

   OpenBtn->Enabled = true;
   BrowseBtn->Enabled = true;
   DecodeBtn->Enabled = true;
   FileName->Enabled = true;
   Label1->Enabled = true;

   delete filenames;
   delete msg;
}
//---------------------------------------------------------------------------

void __fastcall TDecodeWin::FileNameChange(TObject *Sender)
{
   DecodeBtn->Enabled = FileExists(FileName->Text);
   OpenBtn->Enabled = DirectoryExists(ExtractFilePath(FileName->Text));
}
//---------------------------------------------------------------------------

void __fastcall TDecodeWin::Opendirectory1Click(TObject *Sender)
{
   ExploreFolder(ExtractFilePath(FileName->Text));
}
//---------------------------------------------------------------------------

void __fastcall TDecodeWin::File1Click(TObject *Sender)
{
   DecodeBtn->Enabled = FileExists(FileName->Text);
   OpenBtn->Enabled = DirectoryExists(ExtractFilePath(FileName->Text));
}
//---------------------------------------------------------------------------

