/*
*  gpg2.cpp
*
*  Copyright (C) 2010 Richard Christman
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 3 of the License, or (at your
*  option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver Lite, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors
*  of QuickSilver Lite grant you additional permission to convey the
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "gpg2.h"
#include "gpg.h"
#include "global.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
const int bufsize = 32768;
//---------------------------------------------------------------------------

void __fastcall assemblemessagesig(TStrings *message, TStream *output,
   TStrings *headers, TStrings *usenetsig, bool maskversion)
{
   message->LoadFromStream(output);

   // mask gog version
   if (maskversion) // do the masking of the Version Line gpg emits
   {
	  for (int i = 0; i < message->Count; i++)
	  {
		 if (message->Strings[i].Pos("Version:") == 1)
		 {
			message->Strings[i] = "Version: N/A";
			break;
		 }
	  }
   }

   // message headers
   for (int i = 0; i < headers->Count; i++)
	  message->Insert(i, headers->Strings[i]);

   // usenet signature
   if (usenetsig->Count)
   {
	  message->Add("");
	  for (int i = 0; i < usenetsig->Count; i++)
		 message->Add(usenetsig->Strings[i]);
   }
}
//---------------------------------------------------------------------------

void __fastcall getusenetsig(TStrings *usenetsig, TStrings *msg)
{
   int x = msg->IndexOf("-- ");
   if (x > 0)
   {
	  while (msg->Count > x)
	  {
		 usenetsig->Add(msg->Strings[x]);
		 msg->Delete(x);
	  }
   }
}
//---------------------------------------------------------------------------

bool __fastcall GPGClearSignMessage2(TForm *Owner, TStrings *Message,
   const AnsiString &SignKey, AnsiString &Passphrase, bool MaskVersion)
{
   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"PGP signing...");
   AnsiString line, cmdline = getcommandline(GPGclearsign, GPGClearSignCmd);

   TMemoryStream *input = new TMemoryStream();
   TMemoryStream *output = new TMemoryStream();
   TStringList *headers = new TStringList();
   TStringList *usenetsig = new TStringList();

   getusenetsig(usenetsig, Message);
   getheaders(headers, Message);
   trimbodybottom(Message);
   trimbodyright(Message);

   setfield("%signkey%", SignKey, cmdline);

   Message->SaveToStream(input);
   TGPGConsole2 *con = new TGPGConsole2(Owner);
   bool ok = con->Run(cmdline, Passphrase.c_str(), input, output);
   if (ok)
	  assemblemessagesig(Message, output, headers, usenetsig, MaskVersion);
   else
	  ShowGPGError(Owner, con->error, con->status);
   delete con;

   delete usenetsig;
   delete headers;
   delete output;
   delete input;

   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"Done!");
   Snooze(200);
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall GPGVerifyClearSigned2(TForm *Owner, TStrings *Message)
{
   assert(Message->IndexOf(BeginPGPSignedMessage) >= 0);
   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"PGP verifying...");
   AnsiString cmdline = getcommandline(GPGverifyclearsigned, GPGVerifyClearSignedCmd);

   TMemoryStream *input = new TMemoryStream();
   TMemoryStream *output = new TMemoryStream();
   TStringList *headers = new TStringList();
   getheaders(headers, Message);
   Message->SaveToStream(input);

   TGPGConsole2 *con = new TGPGConsole2(Owner);
   bool ok = con->Run(cmdline, NULL, input, output);
   if (ok)
   {
	  FormatSigVerification(con->error);
	  Message->LoadFromStream(output);
	  headers->AddStrings(con->error);
	  headers->Add("*** BEGIN PGP VERIFIED MESSAGE ***");
	  headers->Add("");
	  putheaders(Message, headers);
	  Message->Add("*** END PGP VERIFIED MESSAGE ***");
   }
   else
   {
	  ShowGPGError(Owner, con->error, con->status);
	  putheaders(Message, headers);
   }
   delete con;

   delete headers;
   delete output;
   delete input;

   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"Done!");
   Snooze(200);

   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall GPGDecryptMessage2(TForm *Owner, TStrings *Message,
   AnsiString &Passphrase)
{
   assert(Message->IndexOf(BeginPGPMessage) >= 0);
   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"PGP decrypting...");
   AnsiString cmdline = getcommandline(GPGdecrypt, GPGDecryptCmd);

   TMemoryStream *input = new TMemoryStream();
   TMemoryStream *output = new TMemoryStream();
   TStringList *headers = new TStringList();
   getheaders(headers, Message);
   Message->SaveToStream(input);

   TGPGConsole2 *con = new TGPGConsole2(Owner);
   bool ok = con->Run(cmdline, Passphrase.c_str(), input, output);
   if (ok || con->status->Text.Pos("DECRYPTION_OKAY"))
   {
	  FormatSigVerification(con->error);
	  Message->AddStrings(headers);
	  if (con->error->Count)
	  {
		 headers->AddStrings(con->error);
		 headers->Add("*** BEGIN PGP DECRYPTED/VERIFIED MESSAGE ***");
		 headers->Add("");
	  }
	  else
	  {
		 headers->Add("*** BEGIN PGP DECRYPTED MESSAGE ***");
		 headers->Add("");
	  }
	  Message->LoadFromStream(output);
	  putheaders(Message, headers);
	  if (con->error->Count)
	  {
		 Message->Add("*** END PGP DECRYPTED/VERIFIED MESSAGE ***");
	  }
	  else
	  {
		 Message->Add("*** END PGP DECRYPTED MESSAGE ***");
	  }
	  ok = true;
   }
   else
   {
	  ShowGPGError(Owner, con->error, con->status);
	  putheaders(Message, headers);
   }
   delete con;

   delete headers;
   delete output;
   delete input;

   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"Done!");
   Snooze(200);

   return ok;
}

//---------------------------------------------------------------------------
//   GPGConsole
//---------------------------------------------------------------------------

__fastcall TGPGConsole2::TGPGConsole2(TForm *owner)
   : TObject()
{
   Owner = owner;
   attrib = new TStringList();
   status = new TStringList();
   error = new TStringList();
   statusstream = new TMemoryStream();
   errorstream = new TMemoryStream();
   inputevent = new TEvent(NULL, false, false, "", false);
}
//---------------------------------------------------------------------------

__fastcall TGPGConsole2::~TGPGConsole2()
{
   delete inputevent;
   delete errorstream;
   delete statusstream;
   delete error;
   delete status;
   delete attrib;
   DestroyHandles();
}
//---------------------------------------------------------------------------

void __fastcall CLOSEHandle(HANDLE &handle)
{
   if (handle)
   {
	  CloseHandle(handle);
	  handle = NULL;
   }
}
//---------------------------------------------------------------------------

bool __fastcall TGPGConsole2::CreateHandles(const AnsiString &cmdLine)
{
   TSecurityAttributes sa;
   HANDLE hOutputReadTmp, hErrorReadTmp, hInputWriteTmp, hPassWriteTmp,
	  hAttrReadTmp, hStatReadTmp;

   bool bstatus = cmdLine.Pos("%stat%");
   bool battrib = cmdLine.Pos("%attr%");
   bool bpasswd = cmdLine.Pos("%pass%");

   DestroyHandles();
   sa.nLength = sizeof(sa);
   sa.lpSecurityDescriptor = NULL;
   sa.bInheritHandle = true;

   if (!CreatePipe(&hInputRead, &hInputWriteTmp, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (!CreatePipe(&hOutputReadTmp, &hOutputWrite, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (!CreatePipe(&hErrorReadTmp, &hErrorWrite, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (bstatus && !CreatePipe(&hStatReadTmp, &hStatWrite, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (battrib && !CreatePipe(&hAttrReadTmp, &hAttrWrite, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (bpasswd && !CreatePipe(&hPassRead, &hPassWriteTmp, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (!DuplicateHandle(GetCurrentProcess(), hInputWriteTmp,
	  GetCurrentProcess(), &hInputWrite, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (!DuplicateHandle(GetCurrentProcess(), hOutputReadTmp,
	  GetCurrentProcess(), &hOutputRead, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (!DuplicateHandle(GetCurrentProcess(), hErrorReadTmp,
	  GetCurrentProcess(), &hErrorRead, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (bstatus && !DuplicateHandle(GetCurrentProcess(), hStatReadTmp,
	  GetCurrentProcess(), &hStatRead, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (battrib && !DuplicateHandle(GetCurrentProcess(), hAttrReadTmp,
	  GetCurrentProcess(), &hAttrRead, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (bpasswd && !DuplicateHandle(GetCurrentProcess(), hPassWriteTmp,
	  GetCurrentProcess(), &hPassWrite, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (battrib) CLOSEHandle(hAttrReadTmp);
   if (bstatus) CLOSEHandle(hStatReadTmp);
   if (bpasswd) CLOSEHandle(hPassWriteTmp);
   CLOSEHandle(hOutputReadTmp);
   CLOSEHandle(hErrorReadTmp);
   CLOSEHandle(hInputWriteTmp);

   return true;
}
//---------------------------------------------------------------------------

void __fastcall TGPGConsole2::DestroyHandles()
{
   CLOSEHandle(hInputRead);
   CLOSEHandle(hOutputRead);
   CLOSEHandle(hInputWrite);
   CLOSEHandle(hOutputWrite);
   CLOSEHandle(hErrorRead);
   CLOSEHandle(hErrorWrite);
   CLOSEHandle(hPassRead);
   CLOSEHandle(hPassWrite);
   CLOSEHandle(hAttrRead);
   CLOSEHandle(hAttrWrite);
   CLOSEHandle(hStatRead);
   CLOSEHandle(hStatWrite);
}
//---------------------------------------------------------------------------

void __fastcall TGPGConsole2::SetupFileDescriptors(AnsiString &cmdline)
{
   int p;
   AnsiString fd;

   p = cmdline.Pos("%stat%");
   if (p)
   {
	  fd = AnsiString((int)hStatWrite);
	  setfield("%stat%", fd, cmdline);
   }

   p = cmdline.Pos("%attr%");
   if (p)
   {
	  fd = AnsiString((int)hAttrWrite);
	  setfield("%attr%", fd, cmdline);
   }

   p = cmdline.Pos("%pass%");
   if (p)
   {
	  fd = AnsiString((int)hPassRead);
	  setfield("%pass%", fd, cmdline);
   }
}
//---------------------------------------------------------------------------

void __fastcall TGPGConsole2::PrepareStartupInfo()
{
   ZeroMemory(&startupinfo, sizeof(STARTUPINFO));
   startupinfo.cb = sizeof(STARTUPINFO);
   startupinfo.lpReserved = NULL;
   startupinfo.lpDesktop = NULL;
   startupinfo.lpTitle = NULL;
   startupinfo.lpReserved2 = NULL;
   startupinfo.cbReserved2 = 0;
   startupinfo.dwFlags = STARTF_USESTDHANDLES;
   startupinfo.hStdInput = hInputRead;
   startupinfo.hStdOutput = hOutputWrite;
   startupinfo.hStdError = hErrorWrite;
}
//---------------------------------------------------------------------------

bool __fastcall TGPGConsole2::LaunchChildProcess(const AnsiString &cmdLine)
{
   if (!CreateProcess(
	  NULL,             // pointer to filename
	  cmdLine.c_str(),  // pointer to command line string
	  NULL, 		    // pointer to process security attributes
	  NULL, 		    // pointer to thread security attributes
	  true,  		    // handle inheritance flag
	  CREATE_NO_WINDOW, // creation flags
	  NULL, 		    // pointer to new environment block
	  NULL, 		    // pointer to current directory name
	 &startupinfo,      // pointer to STARTUPINFO
	 &processinfo       // pointer to PROCESS_INFORMATION
   ))
   {
	  ShowError("Error creating GPG process: \n");
	  return false;
   }

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TGPGConsole2::SendPassphrase(char *passphrase)
{
   DWORD byteswrote;
   if (!WriteFile(hPassWrite, passphrase, strlen(passphrase), &byteswrote, NULL))
   {
	  ShowError("Error sending the passphrase to GPG: \n");
	  return false;
   }

   CLOSEHandle(hPassWrite);
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TGPGConsole2::Terminate()
{
   if (running)
	  TerminateProcess(processinfo.hProcess, 0);
}
//---------------------------------------------------------------------------

void __fastcall TGPGConsole2::ShowError(const AnsiString &errormsg)
{
   MsgBox(Owner, errormsg + GetLastErrorStr(), MT_ERROR, MB_OK);
}
//---------------------------------------------------------------------------

bool __fastcall TGPGConsole2::Run(AnsiString cmdLine, char *Passphrase,
   TStream *Input, TStream *Output)
{
   inputstream = Input;
   outputstream = Output;

   if (!CreateHandles(cmdLine))
	  return false;

   SetupFileDescriptors(cmdLine);
   PrepareStartupInfo();

   if (!LaunchChildProcess(cmdLine))
	  return false;

   if (Passphrase && !SendPassphrase(Passphrase))
	  return false;

   running = true;
   try
   {
	  if (inputstream)
		 inputthread = new TStdIOInputThread(this);
	  outputthread = new TStdIOOutputThread(this);

	  CLOSEHandle(hOutputWrite);
	  CLOSEHandle(hStatWrite);
	  CLOSEHandle(hErrorWrite);

	  while (MsgWaitForMultipleObjects(1, &processinfo.hProcess, false,
		 INFINITE, QS_ALLINPUT) == WAIT_OBJECT_0 + 1)
			Application->ProcessMessages();
   }
   __finally
   {
	  if (!GetExitCodeProcess(processinfo.hProcess, &exitcode))
		 ShowError("Error getting GPG exit code: \n");

	  CloseHandle(processinfo.hThread);
	  CloseHandle(processinfo.hProcess);

	  status->LoadFromStream(statusstream);
	  error->LoadFromStream(errorstream);
	  running = false;

	  switch (exitcode)
	  {
		 case 1:
			MsgBox(Owner, "GPG processing error!   ", MT_ERROR, MB_OK);
			return false;
		 case 2:
			if (status->Text.Pos("DECRYPTION_OKAY") == 0)
			   MsgBox(Owner, "GPG command error!   ", MT_ERROR, MB_OK);
			//sm(status->Text);
			//sm(error->Text);
			return false;
	  }
	  return true;
   }
}

//---------------------------------------------------------------------------
// TStdIOInputThread
//---------------------------------------------------------------------------

__fastcall TStdIOInputThread::TStdIOInputThread(TGPGConsole2 *Console)
	: TThread(true)
{
   FreeOnTerminate = true;
   con = Console;
   Resume();
}
//---------------------------------------------------------------------------

void __fastcall TStdIOInputThread::Execute()
{
   HANDLE objects[2];
   objects[0] = (void*)con->processinfo.hProcess;
   objects[1] = (void*)con->inputevent->Handle;
   con->inputevent->SetEvent();

   if (WaitForMultipleObjects(2, objects, false, INFINITE) == WAIT_OBJECT_0 + 1)
   {
	  char buffer[bufsize];
	  DWORD bytesWritten, bytesToWrite;
	  TStream *strm = con->inputstream;

	  strm->Seek(0, soFromBeginning);
	  while(strm->Position < strm->Size)
	  {
		 bytesToWrite = strm->Read(buffer, bufsize);
		 if (!WriteFile(con->hInputWrite, buffer, bytesToWrite, &bytesWritten, NULL))
		 {
			con->ShowError("Error sending input to GPG: \n");
			break;
		 }
	  }
   }

   CLOSEHandle(con->hInputWrite);
}

//---------------------------------------------------------------------------
// TStdIOOutputThread
//---------------------------------------------------------------------------

__fastcall TStdIOOutputThread::TStdIOOutputThread(TGPGConsole2 *Console)
	: TThread(true)
{
   FreeOnTerminate = true;
   con = Console;
   Resume();
}
//---------------------------------------------------------------------------

void __fastcall TStdIOOutputThread::Execute()
{
   DWORD bytesRead;
   char buffer[bufsize];

   while (ReadFile(con->hOutputRead, buffer, bufsize, &bytesRead, NULL) && bytesRead > 0)
	  con->outputstream->Write(&buffer[0], bytesRead);
   con->outputstream->Position = 0;

   while (ReadFile(con->hStatRead, buffer, bufsize, &bytesRead, NULL) && bytesRead > 0)
	  con->statusstream->Write(&buffer[0], bytesRead);
   con->statusstream->Position = 0;

   while (ReadFile(con->hErrorRead, buffer, bufsize, &bytesRead, NULL) && bytesRead > 0)
	  con->errorstream->Write(&buffer[0], bytesRead);
   con->errorstream->Position = 0;
}
//---------------------------------------------------------------------------
