/*
*  mixmaster.h 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#ifndef mixmasterH
#define mixmasterH
#include <system.hpp>
#include <classes.hpp>
#include <forms.hpp>
#include "msgmans.h"
//---------------------------------------------------------------------------
bool __fastcall InitMixmasterLibraries();
void __fastcall FreeMixmasterLibraries();

AnsiString __fastcall MixDir();
AnsiString __fastcall MixExe();
AnsiString __fastcall MixDataDir();
AnsiString __fastcall MixCfg();

AnsiString __fastcall MixmasterVersion();
AnsiString __fastcall OpenSSLVersion();

//bool __fastcall MixCheckSystemReady(TForm *Owner);
bool __fastcall MixEncryptMessage(TForm *Owner, TStrings *msg, TStrings *feedback,
   bool preview = false);

bool __fastcall MixGetRemailerNames(TForm *Owner, TStrings *names);
bool __fastcall MixGetMiddlemanNames(TForm *Owner, TStrings *names);
bool __fastcall MixGetRemailerKLens(TForm *Owner, TStrings *remnames);
void __fastcall MixGetType2List(TStrings *type2);
AnsiString __fastcall MixGetMListLastUpdate();
bool __fastcall MixValidChainHeader(TForm *Owner, TStrings *msg);
bool __fastcall MixPasteMessageHeaders(TStrings *msg, bool ishybrid);
void __fastcall MixGetChainAndCopies(TMsgManager *msgman, AnsiString &chain, int &copies);
bool __fastcall MixHasBrokenChainAfterEncrypt(TStrings *chains, AnsiString &reasonfailed);
bool __fastcall MixHasBrokenChain(TForm *Owner, TStrings *message);
bool __fastcall MixHasDuplicateChainAfterEncrypt(TStrings *chains, AnsiString &reasonfailed);
bool __fastcall MixGetBrokenChains(TStrings *chains);
bool __fastcall MixCheckBrokenChain(AnsiString broken, AnsiString chain, AnsiString &rem1,
   AnsiString &rem2);
int __fastcall MixMixMessage(TStrings *Message, AnsiString &Chain, int copies, bool ispost,
   TStrings *Feedback);
void __fastcall MixGetChainAndCopies(TMsgManager *msgman, AnsiString &chain, int &copies);
bool __fastcall MixCreateMixCfgFile();
void __fastcall MixClearMixmasterPool();
bool __fastcall MixSetMixCfgDummies(bool dummies);
bool __fastcall MixGetMixCfgDummies();
bool __fastcall MixFixSharedOwners();
void __fastcall MixUnFixSharedOwners();

AnsiString __fastcall MixAllPingersFile();
AnsiString __fastcall MixMixRandBinFile();
AnsiString __fastcall MixMixMxFile();
AnsiString __fastcall MixMListTxtFile();
AnsiString __fastcall MixPubringMixFile();
AnsiString __fastcall MixPubringRawFile();
AnsiString __fastcall MixRListTxtFile();
AnsiString __fastcall MixPubringAscFile();
AnsiString __fastcall MixStarexFile();
AnsiString __fastcall MixMixCfgFile();
AnsiString __fastcall MixMixIniFile();
AnsiString __fastcall MixErrLogFile();
AnsiString __fastcall MixMixlibDllFile();
AnsiString __fastcall MixLibeay32DllFile();
AnsiString __fastcall MixLibSSL32DllFile();
AnsiString __fastcall MixPoolDir();

const AnsiString mixmlisttxt = "mlist.txt";
const AnsiString mixpubringmix = "pubring.mix";
const AnsiString mixpubringraw = "pubring.upd";
const AnsiString mixrlisttxt = "rlist.txt";
const AnsiString mixpubringasc = "pubring.asc";
//const AnsiString mixallpingerstxt = "allpingers.txt";
const AnsiString mixallpingers = "allpingers.txt";
const AnsiString allpingers = "allpingers";
const AnsiString mixmixmx = "mix.tls";
const AnsiString mixmixtls = "mixtls";
const AnsiString mixstarextxt = "starex.txt";
const AnsiString mixmixlibdll = "mixlib.dll";
const AnsiString mixlibeay32dll = "libeay32.dll";
const AnsiString mixlibssl32dll = "ssleay32.dll";
const AnsiString mixmixrandbin = "mixrand.bin";
const AnsiString mixtype2list = "type2.list";
const AnsiString MixAllPingersURLs = "https://www.quicksilvermail.net/remailers/allpingers.txt";
const AnsiString MixMixMxURLs  = "https://www.quicksilvermail.net/remailers/mix.tls";
const AnsiString mixmixmxurl  = "mixmxurl";
const AnsiString mixmixpnurl  = "mixpnurl";
const AnsiString MixDefSource = "banana";

const AnsiString MixBeginRemailerMessage = "-----BEGIN REMAILER MESSAGE-----";
const AnsiString MixEndRemailerMessage = "-----END REMAILER MESSAGE-----";

typedef struct {
  byte *data;
  long length;
  long ptr;
  long size;
  byte sensitive;
} BUFFER;

// misc mix
int __fastcall mix_init(char *mixdir);
int __fastcall mix_initex(char *mixdir);
char* __fastcall mix_version();
int __fastcall mix_portable();
void __fastcall mix_exit();
int __fastcall mix_encrypt(int type, BUFFER *message, char *chain, int numcopies,
	 BUFFER *feedback);
int __fastcall pgp_verifyclearsign(BUFFER *msg, BUFFER *siginfo);
int __fastcall mix_send(void);
BUFFER* __fastcall buf_new(void);
int __fastcall buf_free(BUFFER *buf);
int __fastcall buf_append(BUFFER *buffer, byte *mess, int len);
int __fastcall rnd_mouse(UINT i, WPARAM w, LPARAM l);
int __fastcall rnd_bytes(byte *b, int n);
// not exported from mixmaster
// int __fastcall rnd_number(int range);

// pgp functions for pgp unit
int __fastcall pgp_encrypt(int mode, BUFFER *message, BUFFER *encr, BUFFER *sigid,
   BUFFER *pass, char *pubring, char *secring);
int __fastcall pgp_decrypt(BUFFER *message, BUFFER *pass, BUFFER *sig,
   char *pubring, char *secring);
int __fastcall pgp_mailenc(int mode, BUFFER *msg, char *sigid, BUFFER *pass, char *pubring,
   char *secring);

typedef char*(*MIX_VERSION)(void);
typedef int  (*MIX_INITEX)(char*);
typedef int  (*MIX_INIT)(char*);
typedef void (*MIX_EXIT)();
typedef int  (*MIX_ENCRYPT)(int, BUFFER*, char*, int, BUFFER*);

typedef int  (*MIX_SEND)(void);
typedef int  (*BUF_FREE)(BUFFER*);
typedef int  (*BUF_APPEND)(BUFFER*, byte*, int);
typedef int  (*RND_MOUSE)(UINT, WPARAM, LPARAM);
typedef int  (*RND_BYTES)(byte*, int);
//typedef int  (*RND_NUMBER)(int range);
typedef BUFFER* (*BUF_NEW)(void);

typedef int  (*PGP_ENCRYPTBUF)(int, BUFFER*, BUFFER*, BUFFER*, BUFFER*, char*, char*);
typedef int  (*PGP_DECRYPTBUF)(BUFFER*, BUFFER*, BUFFER*, char*, char*);
typedef int  (*PGP_MAILENC)(int, BUFFER*, char*, BUFFER*, char*, char*);
typedef int  (*PGP_LISTKEYSWITHCOLONS)(char*, BUFFER*);
typedef int  (*PGP_CLEARSIGN)(BUFFER*, char*, BUFFER*, char*);

const AnsiString MIXNonPasteHeaders = "Chain:,Sign-On:,Host:,Fcc:"; //To:,Subject:";
const AnsiString MIXnonpasteheaders = "nonpasteheaders";

// ini entry to allow substitution of non-pasted headers
// [advanced]
// nonpasteheaders="to:,cc:,chain:,fcc:,host:,sign-on:,pgp:,pgpbtn:,post:"

#define MSG_MAIL 1
#define MSG_POST 2

// mix.tls fields

const int mxHost = 0;
const int mxPriority = 1;
const int mxTLS = 2;
const int mx587 = 3;
const int mx465 = 4;
const int mx2525 = 5;

extern HINSTANCE hSSLLib;
extern HINSTANCE hMixLib;
//---------------------------------------------------------------------------

#endif
