/*
*  openssl.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
#include "openssl.h"
#include "utility.h"
#include "global.h"
#include "mixmaster.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------

AnsiString __fastcall OpenSSLVersion()
{
   AnsiString ver = ssleay_version(0);
   return "OpenSSL\t\t" + GetTok(1, ver, ' ');
}
//---------------------------------------------------------------------------

bool __fastcall CheckCreateQSPem(TForm *Owner, TSslContext *SslContext)
{
   AnsiString certfile = AppDataDir() + "qs.pem";
   if (!FileExists(certfile))
	  CreateQSPem(Owner, certfile);
   if (!FileExists(certfile))
   {
	  if (MsgBox(Owner,
		 "You've indicated, in the General Options that you want to use the SSL  \n"
		 "certificate 'qs.pem' in my home directory, but the file doesn't exist.  \n"
		 "I tried to create it from an internal copy I have. That failed! I don't  \n"
		 "know why. You might take a look at the help topic, SSL Certificate for  \n"
		 "more info. Would you like to try to continue this send without it?",
		 MT_CONFIRM, MB_YESNO) == IDNO)
	  {
		 return false;
	  }
   }
   else
   {
	  SslContext->SslCertFile = certfile;
   }
   return true;
}
//---------------------------------------------------------------------------

/*bool __fastcall CheckCreateQSPem(TForm *Owner, TSslContext *SslContext)
{
   AnsiString certfile = AppDataDir() + "qs.pem";
   if (!FileExists(certfile))
	  CreateQSPem(Owner, certfile);
   if (!FileExists(certfile))
   {
	  if (MsgBox(Owner,
		 "You've indicated, in the General Options that you want to use the SSL  \n"
		 "certificate 'qs.pem' in my home directory, but the file doesn't exist.  \n"
		 "I tried to create it from an internal copy I have. That failed! I don't  \n"
		 "know why. You might take a look at the help topic, SSL Certificate for  \n"
		 "more info. Would you like to try to continue this send without it?",
		 MT_CONFIRM, MB_YESNO) == IDNO)
	  {
		 return false;
	  }
   }
   else
   {
	  SslContext->SslCertFile = certfile;
   }
   return true;
}*/
//---------------------------------------------------------------------------

void __fastcall CreateQSPem(TForm *Owner, const AnsiString &FileName)
{
   TStringList *cert = new TStringList();
   cert->Add("-----BEGIN RSA PRIVATE KEY-----");
   cert->Add("MIICXgIBAAKBgQDZFBci39tzE5sDKAPsmVafo5jJiRK9eO/xPcNdd18XZQ18h+7Z");
   cert->Add("bNmoRHJFAk8ZOdw4t99v11JlQUQQ0Gd9OnbfxQ1I8MQ2XVwgcqW3CJ+PCGP4Bnn/");
   cert->Add("kBMyRXvRS+9wGTbwDBalrd0b6ZQHyGoTlDC1ABtTyMXFbj7Ex1KoNFvkfwIDAQAB");
   cert->Add("AoGBAKotHsofNhlryDzcNN9ZglgZ12A/DirtUSUM50gxmaFsffsfUXav554LMo24");
   cert->Add("9qtu+Z6ZPxeddKGeXa7UwhyhJahrHBcmDUQX8XPrtH42puKgoWbiVAQviM/0Q/a9");
   cert->Add("GPH8+4ICDHy+fYvbtn+0a7Mp/AnVIab1SjKQS294YANlLT3hAkEA9SDGiS3kBwfx");
   cert->Add("Hdv9MSbJSNmLrj2COjSTE+OFciV8MlAem3Ody00MWmdfT6/ATLiKNOuWrMJyGZ8f");
   cert->Add("tJJxCo0WDwJBAOK01ee/P9mfSnoAlOspx3WvllDgkd3f3DFFfM+pFLlOiNz0dJHD");
   cert->Add("/pd1glgDjyij1QY8I/BxXacEzmgDf8BdOpECQQDucbFuY2t6w2jySLokA7ys1qEf");
   cert->Add("Hsn84INpR1rZzEjhkDoVsQN/bTVlA0UdeLKwbppSHBD3gDYJYlYvCnxXbSJVAkBc");
   cert->Add("tdk4MbT52TOK2wwLc4V3MjxFjOD6HF8e4yzce2fXNC+yNGIrbjyCUfLg7WuxEfa+");
   cert->Add("2v9Th/jloYiLbsrirv2RAkEAiuz82n+Ve5af0MGK3L5R11BhIKVSQiA5C8+aDje4");
   cert->Add("8DH/COzrwKhP2tIvKdG4jx8vk4ayEqR0y1J5RBcojLZEbA==");
   cert->Add("-----END RSA PRIVATE KEY-----");
   cert->Add("-----BEGIN CERTIFICATE-----");
   cert->Add("MIICRzCCAbCgAwIBAgIJANUIyZdXQAo9MA0GCSqGSIb3DQEBBQUAMCIxIDAeBgNV");
   cert->Add("BAMTF3d3dy5xdWlja3NpbHZlcm1haWwubmV0MB4XDTEwMDEzMDIwMzU1NloXDTIw");
   cert->Add("MDEyODIwMzU1NlowIjEgMB4GA1UEAxMXd3d3LnF1aWNrc2lsdmVybWFpbC5uZXQw");
   cert->Add("gZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBANkUFyLf23MTmwMoA+yZVp+jmMmJ");
   cert->Add("Er147/E9w113XxdlDXyH7tls2ahEckUCTxk53Di332/XUmVBRBDQZ306dt/FDUjw");
   cert->Add("xDZdXCBypbcIn48IY/gGef+QEzJFe9FL73AZNvAMFqWt3RvplAfIahOUMLUAG1PI");
   cert->Add("xcVuPsTHUqg0W+R/AgMBAAGjgYQwgYEwHQYDVR0OBBYEFI3R4qMvunrtlz5S2ybc");
   cert->Add("niB5EmjbMFIGA1UdIwRLMEmAFI3R4qMvunrtlz5S2ybcniB5EmjboSakJDAiMSAw");
   cert->Add("HgYDVQQDExd3d3cucXVpY2tzaWx2ZXJtYWlsLm5ldIIJANUIyZdXQAo9MAwGA1Ud");
   cert->Add("EwQFMAMBAf8wDQYJKoZIhvcNAQEFBQADgYEAq2ucPg17l4wW6U04JRCZ2sACvbXG");
   cert->Add("Kc/b32Mr8lvLtYBDbqb6es3G6nqLoOyF4BMRaz5Lpn2TkdnE2DDbFSYDfAhLqpZV");
   cert->Add("ZfMjg22oyd+Pnb/fvxRRdsEnUxLoLLQXzMIWiP8vrPJNWP0zUkZVv/sz/WBk5TfI");
   cert->Add("cdwF3Ti663C33e4=");
   cert->Add("-----END CERTIFICATE-----");
   cert->Add("");
   try
   {
      cert->SaveToFile(FileName);
   }
   catch(Exception &e)
   {
      MsgBox(Owner,
         "I keep a demo client SSL certicicate inside my program. I just tried  \n"
         "to save it as: " + AppDataDir() + "qs.pem, but I wasn't successful!  \n"
         "The error reported was: " + e.Message,
         MT_ERROR, MB_OK);
   }
   delete cert;
}
//---------------------------------------------------------------------------

char* __fastcall ssleay_version(int t)
{
   ::SSLEAY_VERSION version = (::SSLEAY_VERSION)GetProcAddress(hSSLLib, "SSLeay_version");
   assert(version != NULL);
   return version(t);
}
//---------------------------------------------------------------------------

