/*
*  pgpactdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "pgpactdlgs.h"
#include "gpg.h"
#include "utility.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPGPactDlg *PGPactDlg;
char *decryptingdatabase = "...decrypting database...";
char *encryptingdatabase = "...encrypting database...";
//---------------------------------------------------------------------------

bool __fastcall ShowDecryptingDatabase(TQSDB *DB, AnsiString &Passphrase)
{
   TPGPactDlg *pgpactdlg = new TPGPactDlg(Screen->ActiveForm, DB, Passphrase, false);
   pgpactdlg->ShowModal();
   bool ok = pgpactdlg->ModalResult == mrOk;
   delete pgpactdlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall ShowEncryptingDatabase(TQSDB *DB, AnsiString &Passphrase)
{
   TPGPactDlg *pgpactdlg = new TPGPactDlg(Screen->ActiveForm, DB, Passphrase, true);
   pgpactdlg->ShowModal();
   bool ok = pgpactdlg->ModalResult == mrOk;
   delete pgpactdlg;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TPGPactDlg::TPGPactDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TPGPactDlg::TPGPactDlg(TComponent* Owner, TQSDB *DB,
   AnsiString &Passphrase, bool Encrypt) : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   encrypt = Encrypt;
   if (encrypt)
      workingmsg = "...encrypting database...";
   else
      workingmsg = "...decrypting database...";
   Label1->Caption = workingmsg;
   passphrase = Passphrase;
   db = DB;
}
//---------------------------------------------------------------------------

void __fastcall TPGPactDlg::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   Timer2->Enabled = true;
   DWORD ticks = GetTickCount();
   if (encrypt)
   {
      if (!GPGEncryptDatabase(db, passphrase))
      {
         Label1->Caption = "FAILED!", Snooze(200);
         ModalResult = mrCancel;
         return;
      }
   }
   else
   {
      if (!GPGDecryptDatabase(db, passphrase))
      {
         Label1->Caption = "FAILED!", Snooze(200);
         ModalResult = mrCancel;
         return;
      }
   }

   while (GetTickCount() < ticks + 300)
      Snooze(100);

   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TPGPactDlg::FormShow(TObject *Sender)
{
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TPGPactDlg::Timer2Timer(TObject *Sender)
{
   Label1->Caption = Label1->Caption == "" ? workingmsg : AnsiString("");
}
//---------------------------------------------------------------------------

void __fastcall TPGPactDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   Timer2->Enabled = false;
}
//---------------------------------------------------------------------------
