/*
*  pgpdpassdlgs2.cpp
*
*  Copyright (C) 2012 Richard Christman
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 3 of the License, or (at your
*  option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "pgpdpassdlgs2.h"
#include "utility.h"
#include "global.h"
#include "help.h"
#include "dsadlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPGPDPassDlg *PGPDPassDlg;
//---------------------------------------------------------------------------

AnsiString __fastcall GetNymDecryptionPassphrase(TForm *Owner, TStrings *EncryptKeys, bool ThrowKeyIDs)
{
   AnsiString password;
   TPGPDPassDlg *pgpdpassdlg = new TPGPDPassDlg(Owner);
   if (ThrowKeyIDs)
   {
	  pgpdpassdlg->Caption =
		 "Decrypting NYM Message Public-Key Layer";
	  pgpdpassdlg->Label1->Caption =
		 "&Message should be encrypted to this key";
	  pgpdpassdlg->Label3->Caption =
		 "The key shown above is the expected key of a throw-keyids "
		 "encrypted message.";
	  pgpdpassdlg->KeyBox->Lines->Text = EncryptKeys->Text;
	  pgpdpassdlg->Label4->Font->Color = clHotLight;
	  pgpdpassdlg->Label4->Visible = true;
	  pgpdpassdlg->ShowModal();
	  if (pgpdpassdlg->ModalResult == mrOk)
		 password = pgpdpassdlg->Phrase->Text;
   }
   else
   {
	  pgpdpassdlg->Caption =
		 "Decrypting NYM Message Public-Key Layer";
	  pgpdpassdlg->Label1->Caption =
		 "&Message is encrypted with this key";
	  pgpdpassdlg->Label3->Caption =
		 "This is the public-key encrypted, nymserver signed layer "
		 "of your NYM message.";
	  pgpdpassdlg->KeyBox->Lines->Text = EncryptKeys->Text;
	  pgpdpassdlg->Label4->Visible = false;
	  pgpdpassdlg->ShowModal();
	  if (pgpdpassdlg->ModalResult == mrOk)
		 password = pgpdpassdlg->Phrase->Text;
   }
   delete pgpdpassdlg;
   return password;
}
//---------------------------------------------------------------------------

AnsiString __fastcall GetNymSymmetricPassphrase(TForm *Owner, TStrings *EncryptKeys)
{
   AnsiString password;
   TPGPDPassDlg *pgpdpassdlg = new TPGPDPassDlg(Owner);
   pgpdpassdlg->Caption =
	  "Decrypting NYM Message Symmetric Layer";
   pgpdpassdlg->Label1->Caption =
	  "&Message is encrypted with a passphrase";
   pgpdpassdlg->Label3->Caption =
	  "This is a symmetrically encrypted layer of your NYM message.";
   pgpdpassdlg->KeyBox->Lines->Text = EncryptKeys->Text;
   pgpdpassdlg->Label4->Visible = false;
   pgpdpassdlg->ShowModal();
   if (pgpdpassdlg->ModalResult == mrOk)
	  password = pgpdpassdlg->Phrase->Text;
   delete pgpdpassdlg;
   return password;
}
//---------------------------------------------------------------------------

AnsiString __fastcall GetDecryptionPassphrase(TForm *Owner, TStrings *EncryptKeys)
{
   AnsiString password;
   TPGPDPassDlg *pgpdpassdlg = new TPGPDPassDlg(Owner);
   pgpdpassdlg->Caption =
	  "Decrypting PGP Encrypted Message";
   pgpdpassdlg->Label1->Caption =
	  "&Message is encrypted with these keys";
   pgpdpassdlg->Label3->Visible = false;
   pgpdpassdlg->KeyBox->Lines->Text = EncryptKeys->Text;
   pgpdpassdlg->Height -= 35;
   pgpdpassdlg->ShowModal();
   if (pgpdpassdlg->ModalResult == mrOk)
	  password = pgpdpassdlg->Phrase->Text;
   delete pgpdpassdlg;
   return password;
}
//---------------------------------------------------------------------------

__fastcall TPGPDPassDlg::TPGPDPassDlg(TComponent* Owner)
		: TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   Phrase->Clear();
   KeyBox->Clear();
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Phrase->Font, ini->ReadString("settings", "editorfont", OptsDefEditorFont));
   AssignFont(KeyBox->Font, ini->ReadString("settings", "windowfont", OptsDefWindowFont));
   EditColor = Phrase->Font->Color;
   Phrase->Font->Color = clWindow;
   delete ini;
}
//---------------------------------------------------------------------------

__fastcall TPGPDPassDlg::~TPGPDPassDlg()
{
   WipeEditBox(Phrase);
}
//---------------------------------------------------------------------------

void __fastcall TPGPDPassDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult == mrOk && Phrase->Text.IsEmpty())
   {
	  MsgBox(this,
		 "You need enter a passphrase--or click Cancel!",
		  MT_INFO, MB_OK);
	  CanClose = false;
	  return;
   }
}
//---------------------------------------------------------------------------

void __fastcall TPGPDPassDlg::HideBoxClick(TObject *Sender)
{
   if (!HideBox->Checked)
	  Phrase->Font->Color = EditColor;
   else
      Phrase->Font->Color = clWindow;
}
//---------------------------------------------------------------------------

void __fastcall TPGPDPassDlg::FormShow(TObject *Sender)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TPGPDPassDlg::Label4Click(TObject *Sender)
{
	MsgBox(this,
	  "What is Throw-KeyIDs?\n\n"
	  "When GPG encrypts a message it includes the Key ID of the key used. "
	  "This way, when the message is decrypted, GPG looks for the Key ID so "
	  "it knows which key to use for the decryption task. However, GPG "
	  "has a feature that allows it to encrypt the message without including "
	  "the Key ID. It's called throw-keyids. The point is to hide which key "
	  "encrypted the message. It's an additional security measure used "
	  "occasionally to help foil attackers. In order for GPG to decrypt the "
	  "message, it tries each secret key on the users keyring until the "
	  "correct key succeeds. Currently, we have an "
	  "experimental type of nymserver that uses this tactic. If you see a "
	  "reference to throw-keyids then you're using this new type of nymserver.",
	   MT_INFO, MB_OK);

}
//---------------------------------------------------------------------------

void __fastcall TPGPDPassDlg::Label4MouseEnter(TObject *Sender)
{
   Label4->Font->Color = clHighlight;
}
//---------------------------------------------------------------------------

void __fastcall TPGPDPassDlg::Label4MouseLeave(TObject *Sender)
{
   Label4->Font->Color = clHotLight;
}
//---------------------------------------------------------------------------

