/*
*  pgpspassdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "PGPspassDlgs.h"
#include "gpg.h"
#include "utility.h"
#include "global.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//TPGPspassDlg *PGPspassDlg;
//---------------------------------------------------------------------------

AnsiString __fastcall GetSecretKeyPassword(TForm *Owner, AnsiString &signkey)
{
   AnsiString phrase;
   TPGPspassDlg *pgpspassdlg = new TPGPspassDlg(Owner, signkey);
   if (pgpspassdlg->ShowModal() == mrOk)
   {
      signkey = GetTok(1, pgpspassdlg->KeyBox->Items->Strings[pgpspassdlg->KeyBox->ItemIndex], ':');
      assert(signkey != "");
      phrase = pgpspassdlg->Phrase->Text;
   }
   delete pgpspassdlg;
   return phrase;
}
//---------------------------------------------------------------------------

__fastcall TPGPspassDlg::TPGPspassDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TPGPspassDlg::TPGPspassDlg(TComponent* Owner, AnsiString SignKey)
        : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   Phrase->Clear();
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Phrase->Font, ini->ReadString("settings", "editorfont", OptsDefEditorFont));
   AssignFont(KeyBox->Font, ini->ReadString("settings", "windowfont", OptsDefWindowFont));
   EditColor = Phrase->Font->Color;
   //KeyBox->Font->Size = 8;
   delete ini;
   KeyBox->ItemHeight = KeyBox->Font->Height + 4;

   TGPG *pgp = new TGPG();
   if (pgp->CheckInitSuccess())
   {
      if (SignKey.IsEmpty())
         SignKey = pgp->defaultkey;

      TStringList *keys = new TStringList();
      keys->Sorted = true;

      pgp->ListSecretKeysWithColons(keys);
      for (int i = 0; i < keys->Count; i++)
      {
         AnsiString keyid, key = keys->Strings[i];
         if (pgp->KeyType(key) != "sec" || pgp->KeyValid(key) == "e")
            continue;

		 keyid = pgp->KeyID8(key);
		 KeyBox->Items->Add(pgp->KeyUID(key) + ":0x" + keyid);
	  }
	  for (int i = 0; i < KeyBox->Items->Count; i++)
	  {
		 if (KeyBox->Items->Strings[i].LowerCase().Pos(":" + SignKey.LowerCase()) > 0)
		 {
			KeyBox->ItemIndex = i;
			break;
		 }
	  }
   }
   delete pgp;
   HideBoxClick(HideBox);
   keyidwidth = KeyBox->Canvas->TextWidth("0xXXXXXXXXX");
}

//---------------------------------------------------------------------------

__fastcall TPGPspassDlg::~TPGPspassDlg()
{
   WipeEditBox(Phrase);
}
//---------------------------------------------------------------------------

void __fastcall TPGPspassDlg::HideBoxClick(TObject *Sender)
{
   if (!HideBox->Checked)
	  Phrase->Font->Color = EditColor;
   else
	  Phrase->Font->Color = clWindow;
}
//---------------------------------------------------------------------------

void __fastcall TPGPspassDlg::FormCloseQuery(TObject *Sender,
	  bool &CanClose)
{
   if (ModalResult == mrOk)
   {
	  if (Phrase->Text.IsEmpty() || KeyBox->Text.IsEmpty())
	  {
		 MsgBox(this,
			"You must select a key and enter a passphrase--or click Cancel!",
			 MT_INFO, MB_OK);
		 CanClose = false;
		 return;
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall TPGPspassDlg::KeyBoxDrawItem(TWinControl *Control,
	  int Index, TRect &Rect, TOwnerDrawState State)
{
   AnsiString id, user, item = KeyBox->Items->Strings[Index];
   id = GetTok(1, item, ':');
   user = GetTok(0, item, ':');

   KeyBox->Canvas->TextRect(Rect, Rect.left + 2, Rect.top + 1, id);
   Rect.left += keyidwidth + 10;
   KeyBox->Canvas->TextRect(Rect, Rect.Left + 2, Rect.top + 1, user);
}
//---------------------------------------------------------------------------

